/**
* File         : BitArray.h
* Author       : Zeynep Odcikin-Ozdemir
* Created      : Friday Feb 20 09:23:06 1998
* Revision     : $Revision$
* Updated      : $Date$
*/

#ifndef _BITARRAY_H
#define _BITARRAY_H


#include <iostream.h>
#include <stdio.h>

enum  Boolean { TRUE = 1, FALSE=0 } ;
char  toHex[] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
		  'A', 'B', 'C', 'D', 'E', 'F' };

template<class T>
class BitArray 
{
  
   public:

     /** Constructor */
     BitArray();

     /** Construct a bit array of size sz */
     BitArray(size_t sz);

     /** Copy Constructor */ 
     BitArray(BitArray<T>& RHS);

     /** Destructor for BitArray Object */
     ~BitArray();

     void set(size_t indx);

     void reset(size_t indx);

     Boolean test(size_t indx) const;

     size_t BitArray::count() const;

     size_t getNumberOfElements() const ;

     size_t getNumberOfBits() const ;

     BitArray<T> HexStringToBitArray(char *buf, int s) const; 

     BitArray<T> IntToBitArray(unsigned int val, int N) const; 

     BitArray<T> IntToBitArray(unsigned int val) const; 

     unsigned int toInt() const ; 

     BitArray<T> shuffle(int *permutation, int sz); 

     BitArray<T> copyFrom(size_t from, size_t numberOfBits); 

     BitArray<T> cyclicCopyFrom(size_t from, size_t numberOfBits); 

     /** Assignment Operator */
     BitArray<T> operator=(const BitArray<T>& RHS); 

     /** And  Operator */
     BitArray<T> operator&(const BitArray<T>& RHS); 

     /** Or  Operator */
     BitArray<T> operator|(const BitArray<T>& RHS); 

     /** XOR  Operator */
     BitArray<T> operator^(const BitArray<T>& RHS); 

     BitArray<T> operator+(const BitArray<T>& RHS); 

     /** Left-Shift  Operator */
     BitArray<T> operator<<(size_t n); 

     /** Right-Shift  Operator */
     BitArray<T> operator>>(size_t n); 

     /** Overwrite the stream output operation */
     friend ostream& operator<<(ostream& os,  const BitArray<T>& RHS)
     {
         RHS.printMe(os);
	 return os;
     }; 

     /** Overwrite the stream input operation */
     friend istream& operator>>(istream& is,  BitArray<T>& RHS);

     void printMeClean(ostream& os) const ;
     void printBinary(ostream& os) const ;

   protected:

     /** Reclaims the memory allocated before for this object */
     void dispose();

     /** Copies the object to this object */
     void copyFrom(const BitArray<T>& RHS);

     /** Prinst the object to the osutput stream */
     void printMe(ostream& os) const ;

   private:

     enum { BitsPerByte = 8,
	    NumBitsPerElement = sizeof(T) * BitsPerByte };

     size_t  size;          /** Maximum number of bits */

     size_t  numElements ;  /** Number of elements in array */  

     T *bits;         /** Array of bits */ 

};

#endif _BITARRAY_H


